' ================================================================================================
' - ENGLISH - TEMPERATURE TRIGGER MULTIPLE SLOTS
' ================================================================================================
'  This script reads a specified number of temperature sensors
'  If a sensor temperature is more then TriggerTemperature
'   a high signal (1000) is emitted for 100 mS on the corresponding output slot
'  Before to emit another signa the trigger must be "rearmed" with a low temperature
'  The sensor LM35 (3 Euro) is factory trimmed with great precision (0.25 degree)
' ------------------------------------------------------------------------------------------------

' ================================================================================================
' - ITALIANO - TEST DI TEMPERATURA A SOGLIA - PER SLOT MULTIPLI
' ================================================================================================
'  Questo script legge un certo numero di sensori di temperatura
'  Se la temperatura di un sensore supera la TriggerTemperature 
'   viene emesso un segnale alto (1000) per 100 mS, sul corrisponedente slot di uscita
'  Prima di emettere un nuovo segnale il sistema deve venire "riarmato" con temperatura bassa
'  Il sensore LM35 (3 Euro) è calibrato in fabbrica con precisione (0.25 gradi)
' ------------------------------------------------------------------------------------------------

Const TriggerTeperature As Single = 30
Const FirstInputSlot As Int32 = 1
Const FirstOutputSlot As Int32 = 11
Const NumSlots As Int32 = 5

Dim OldTemp(NumSlots -1) As Single
Dim NewTemp(NumSlots -1) As Single

Sub Main(ByVal s() As String)
	If Form1 IsNot Nothing Then Return
	Form1 = New Form
	InitializeComponents
	MemoryMappedFile_Init
	While Form1.Visible
		Application.DoEvents
		Threading.Thread.Sleep(10)
	End While
End Sub

Sub Form1_Closing(sender As Object, e As CancelEventArgs) Handles Form1.Closing
	Timer1.Stop
End Sub

Sub Timer1_Tick(ByVal sender As Object, ByVal e As EventArgs) Handles Timer1.Tick
	Static Temp As Single
	For i As Int32 = 0 To NumSlots -1
		' ----------------------------------------------------- calc the temperature
		Temp = ReadSlot(FirstInputSlot + i) / 3
		' ----------------------------------------------------- smooth the temperature
		SmoothValue_Pow_Adaptive(NewTemp(i), Temp, 10)
		' ----------------------------------------------------- test trigger
		If NewTemp(i) >= TriggerTeperature And OldTemp(i) < TriggerTeperature Then
			WriteSlot(FirstOutputSlot + i, 1000)
			System.Threading.Thread.Sleep(100)
			WriteSlot(FirstOutputSlot + i, 0)
		End If
		' ----------------------------------------------------- update old values
		OldTemp(i) = NewTemp(i)
		' ----------------------------------------------------- show first slot temperature
		Label1.Text = NewTemp(0).Tostring("0.000")
	Next
End Sub


' ================================================================================================
'   An adaptive, HiQuality, smoothing function
' ================================================================================================
Private Sub SmoothValue_Pow_Adaptive(ByRef value As Single, ByVal new_value As Single, ByVal speed As Single)
	Dim delta As Single = new_value - value
	Dim delta_sgn As Single = Math.Sign(delta)
	Dim delta_abs As Single = Math.Abs(delta)
	Dim delta_pow As Single = CSng(delta_abs ^ 2) * speed
	If value <> 0 Then delta_pow /= abs(value)
	If delta_pow >= delta_abs Then
		value = new_value
	Else
		value += delta_sgn * delta_pow
	End If
End Sub

' ================================================================================================
'   User Interface Components
' ================================================================================================
WithEvents Form1 As Form
WithEvents Timer1 As Timer = New Timer
Dim Label1 As Label = New Label

Sub InitializeComponents()
	Application.EnableVisualStyles
	form1.SuspendLayout
	' ------------------------------------------------------------------ Form
	Form1.Text = "Theremino Script - Temp Trigger"
	Form1.FormBorderStyle = FormBorderStyle.Fixed3D
	Form1.MaximizeBox = False
	Form1.ClientSize = New System.Drawing.Size(320, 90)
	Form1.MinimumSize = Form1.Size
	Form1.StartPosition = FormStartPosition.CenterScreen
	' ------------------------------------------------------------------ Label 1
	'Label1.BorderStyle = BorderStyle.Fixed3D
	Label1.Location = New Point(0, 0)
	Label1.Size = Form1.ClientSize
	Label1.BackColor = Color.DarkBlue
	Label1.ForeColor = Color.Gold
	Label1.Font = New Font("Tahoma", 55, FontStyle.Bold)
	Label1.TextAlign = ContentAlignment.MiddleRight
	Label1.Anchor = AnchorStyles.Top Or AnchorStyles.Left Or _
					AnchorStyles.Right Or AnchorStyles.Bottom
	Form1.Controls.Add(Label1)
	' ------------------------------------------------------------------ Timer 1
	Timer1.Interval = 15
	Timer1.Start
	' ------------------------------------------------------------------ 
	Form1.ResumeLayout
	Form1.PerformLayout()
	Form1.Show
End Sub
